from django import forms

import bleach


class SanitizedCharField(forms.CharField):
    """
    A subclass of CharField that escapes (or strip) HTML tags and attributes.
    """    
    def __init__(self, allowed_tags=[], allowed_attributes=[], 
            allowed_styles=[], strip=False, *args, **kwargs):
        self._allowed_tags = allowed_tags
        self._allowed_attributes = allowed_attributes
        self._allowed_styles = allowed_styles
        self._strip = strip
        super(SanitizedCharField, self).__init__(*args, **kwargs)

    def clean(self, value):
        value = super(SanitizedCharField, self).clean(value)
        return bleach.clean(value, tags=self._allowed_tags,
            attributes=self._allowed_attributes, 
            styles=self._allowed_styles, strip=self._strip)



from django import forms
from django.db import models
from django.test import TestCase
from django.test.utils import override_settings

from sanitizer.templatetags.sanitizer import (sanitize, sanitize_allow,
    escape_html, strip_filter, strip_html)
from .forms import SanitizedCharField as SanitizedFormField
from .models import SanitizedCharField, SanitizedTextField


ALLOWED_TAGS = ['a']
ALLOWED_ATTRIBUTES = ['href', 'style']
ALLOWED_STYLES = ['width']


class TestingModel(models.Model):
    test_field = SanitizedCharField(max_length=255, allowed_tags=ALLOWED_TAGS, 
        allowed_attributes=ALLOWED_ATTRIBUTES, allowed_styles=ALLOWED_STYLES)


class TestingTextModel(models.Model):
    test_field = SanitizedTextField(allowed_tags=ALLOWED_TAGS, 
        allowed_attributes=ALLOWED_ATTRIBUTES, allowed_styles=ALLOWED_STYLES)


class TestForm(forms.Form):
    test_field = SanitizedFormField(allowed_tags=['a'], 
    allowed_attributes=['href', 'style'], allowed_styles=['width'])


class SanitizerTest(TestCase):

    @override_settings(SANITIZER_ALLOWED_TAGS=['a'])
    def test_sanitize(self):
        """ Test sanitize function in templatetags """
        self.assertEqual(sanitize('test<script></script>'), 
            'test&lt;script&gt;&lt;/script&gt;')

    def test_strip_filter(self):
        """ Test strip_html filter """
        self.assertEqual(strip_filter('test<script></script>'), 'test')

    def test_sanitize_allow(self):
        """ Test sanitize_allow function in templatetags """
        self.assertEqual(sanitize_allow('test<script></script><br>', 'br'), 'test<br>')
        self.assertEqual(sanitize_allow('test<script></script><br/>', 'br'), 'test<br>')
        self.assertEqual(sanitize_allow('<a href="">test</a>', 'a'), '<a>test</a>')
        self.assertEqual(sanitize_allow('<a href="">test</a>', 'a; href'), '<a href="">test</a>')


    def test_SanitizedCharField(self):
        TestingModel.objects.create(test_field='<a href="" style="width: 200px; height: 400px">foo</a><em>bar</em>')
        test = TestingModel.objects.latest('id')
        self.assertEqual(test.test_field, '<a href="" style="width: 200px;">foo</a>&lt;em&gt;bar&lt;/em&gt;')


    def test_SanitizedTextField(self):
        TestingTextModel.objects.create(test_field='<a href="" style="width: 200px; height: 400px">foo</a><em>bar</em>')
        test = TestingTextModel.objects.latest('id')
        self.assertEqual(test.test_field, '<a href="" style="width: 200px;">foo</a>&lt;em&gt;bar&lt;/em&gt;')

    def test_SanitizedFormField(self):
        html = '<a href="" style="width: 200px; height: 400px">foo</a><em class=""></em>'
        form = TestForm({ 'test_field': html })
        form.is_valid()
        self.assertEqual(form.cleaned_data['test_field'],
                         '<a href="" style="width: 200px;">foo</a>&lt;em class=""&gt;&lt;/em&gt;')

    def test_escape_html(self):
        html = '<a href="" class="" style="width: 200px; height: 400px">foo</a><em></em>'
        self.assertEqual(escape_html(html, allowed_tags='a', 
            allowed_attributes='href,style', allowed_styles='width'),
            '<a href="" style="width: 200px;">foo</a>&lt;em&gt;&lt;/em&gt;')
        self.assertEqual(escape_html(html, allowed_tags=['a'], 
            allowed_attributes=['href', 'style'], allowed_styles=['width']),
            '<a href="" style="width: 200px;">foo</a>&lt;em&gt;&lt;/em&gt;')
    
    def test_strip_html(self):
        html = '<a href="" class="" style="width: 200px; height: 400px">foo</a><em></em>'
        self.assertEqual(strip_html(html, allowed_tags='a', 
            allowed_attributes='href,style', allowed_styles='width'),
            '<a href="" style="width: 200px;">foo</a>')
        self.assertEqual(strip_html(html, allowed_tags=['a'], 
            allowed_attributes=['href', 'style'], allowed_styles=['width']),
            '<a href="" style="width: 200px;">foo</a>')



from django import forms

import bleach


def get_sanitized_clean_func(original_clean, **kwargs):
    def fn(value):
        value = original_clean(value)
        if isinstance(value, basestring):
            value = bleach.clean(value, **kwargs)
        return value
    return fn


class sanitize(object):

    
    def __init__(self, tags=bleach.ALLOWED_TAGS,
                 attributes=bleach.ALLOWED_ATTRIBUTES, styles=[], strip=False,
                 strip_comments=True):
        self.kwargs = {
            'tags': tags,
            'attributes': attributes,
            'styles': styles,
            'strip': strip,
            'strip_comments': strip_comments,
        }


    def __call__(self, cls):
        self.actual_decorator(cls)
        return cls
        
        
    def actual_decorator(self, cls):
        fields = [(key, value) for key, value in cls.base_fields.iteritems() if isinstance(value, forms.CharField)]
        for field_name, field_object in fields:
            original_clean = getattr(field_object, 'clean')
            clean_func = get_sanitized_clean_func(original_clean, **self.kwargs)
            setattr(field_object, 'clean', clean_func)
